/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealStereoElecDriver.h"
#include "StereoElecDriver.h"
#include "StereoElecDef.h"

#include <iostream>
#include <boost/thread.hpp>
#include <boost/bind.hpp>


using namespace std;

namespace stereo_elec {


const int32 RealStereoElecDriver::READ_COMMAND = 1;
const int32 RealStereoElecDriver::START_COMMAND = 2;
const int32 RealStereoElecDriver::STOP_COMMAND = 3;
const int32 RealStereoElecDriver::SHUTDOWN_COMMAND = 4;
const int32 RealStereoElecDriver::READINFO_COMMAND = 5;


/*
 * Constructor
 */
RealStereoElecDriver::RealStereoElecDriver(StereoElecDriver* owner) :
		StereoElecState(owner) {


    m_commandQueue = new common::Queue<int32>(1024);
    m_serverThread = new boost::thread(boost::bind(&RealStereoElecDriver::qtLoop, this));

	firststart=true;
}

/*
 * Destructor
 */
RealStereoElecDriver::~RealStereoElecDriver() {


	m_commandQueue->put(new int32(SHUTDOWN_COMMAND));
	m_serverThread->join();
	delete m_serverThread;
	delete m_commandQueue;

}

void RealStereoElecDriver::qtLoop() {

	int argc = 0;
	char **argv = 0;
	QCoreApplication qtApplication(argc, argv);

	TstereoDaqLib* daq=new TstereoDaqLib();
	while(true){
        int32 * command = m_commandQueue->get();
        if (*command == SHUTDOWN_COMMAND) {
        	delete daq;
        	break;

		} else if (*command == START_COMMAND) {
			if(firststart){
				daq->createTrigEnMask(owner()->DAQParams);

				cout << "createTrigEnMask ok" << endl;

				daq->InitAMCProto(owner()->nfeboard());
				cout << owner()->nfeboard() << endl;
				cout << owner()->DAQParams.FEParams_v.size() << endl;

				daq->DoConfig(owner()->DAQParams);
				cout << "DoConfig ok" << endl;
				firststart=false;
			}
			daq->DoStart(owner()->DAQParams);
			cout << "DoStart ok" << endl;
			daq->preQtAcqLoop();
			m_TimeCounter.clear();

		}
		else if (*command == READ_COMMAND) {
			daq->QtAcqLoop();
		}
		else if (*command == READINFO_COMMAND) {
			unsigned int accepted_cnt,rejected_cnt;
			daq->QtAcqLoopCounter(accepted_cnt,rejected_cnt,false);
			cout << "Trig accepted " << accepted_cnt << "rejected " << rejected_cnt << endl;
		}else if (*command == STOP_COMMAND) {

			cout << "stop " << endl;
			daq->DoStop();

			cout << "save " << endl;
			daq->DoSave();
		}

        delete command;
	}
}
/*
 * init
 */
void RealStereoElecDriver::init() {


}

/*
 * clear
 */
void RealStereoElecDriver::clear() {

}

/*
 * writeParam
 */
void RealStereoElecDriver::writeParam() {

}


/*
 * readInfos
 */
void RealStereoElecDriver::readInfos() {

	if(owner()->startActivated()) m_commandQueue->put(new int32(READINFO_COMMAND));
}
/*
 * start
 */
void RealStereoElecDriver::start() {
	m_commandQueue->put(new int32(START_COMMAND));
	/*
	if(firststart){
		daq->createTrigEnMask(owner()->DAQParams);
		cout << "createTrigEnMask ok" << endl;

		daq->InitAMCProto(owner()->nfeboard());
		cout << owner()->nfeboard() << endl;
		cout << owner()->DAQParams.FEParams_v.size() << endl;
		daq->DoConfig(owner()->DAQParams);
		cout << "DoConfig ok" << endl;
		firststart=false;
	}
	daq->DoStart(owner()->DAQParams);
	cout << "DoStart ok" << endl;
	daq->preQtAcqLoop();
	m_TimeCounter.clear();
	cout << "test start " << endl;
	//m_Status = COUNT_ON;*/
}

/*
 * stop
 */
void RealStereoElecDriver::stop() {
	 m_commandQueue->put(new int32(STOP_COMMAND));
}

/*
 * read
 */
void RealStereoElecDriver::read() {

	if(owner()->startActivated()) m_commandQueue->put(new int32(READ_COMMAND));

	/*

	uint32 size = (uint32) owner()->dataSize();
	int32* data = owner()->data();

	// Calculate the detector sum
	float64 sum = 0;
	for (uint32 i = 0; i < size; i++) {
		// Limit value for avoid negative value

		// Calculate the sum
		sum += (float64) data[i];
	}

	// If data changed send event
	owner()->sum.update(sum);
	owner()->data.sendEvent();*/
}

/*
 * synchroniseRead
 */
void RealStereoElecDriver::synchroniseRead() {
	read();
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * readStatus
 */
void RealStereoElecDriver::readStatus() {
	if (owner()->startActivated() == true) {
		bool finished = false;
		int32 mode = owner()->mode();

		float64 actual = m_TimeCounter.getTime();
		owner()->time.update(actual);

		if (mode == acquisition::GateCommon::TIME_MODE) {
			float64 target = owner()->time.setpoint();
			if (actual >= target) {
				finished = true;
				actual = target;
			}

		} else if (mode == acquisition::GateCommon::COUNTER1_MODE) {

		}

		if (finished) {
			//m_Status = COUNT_END;
			//owner()->status.update(m_Status);
			//owner()->computeGateStatus(m_Status);
			//owner()->computeAcquisitionStatus(0);
			owner()->computeDetectorStatus(0);
			owner()->startActivated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		} else {
			//owner()->status.update(m_Status);
			//owner()->computeGateStatus(m_Status);
			//owner()->computeAcquisitionStatus(0);
			owner()->computeDetectorStatus(0);
			owner()->calculateProgression();
		}
	} else {
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
		//m_Status = COUNT_END;
		//owner()->status.update(m_Status);
		//owner()->computeGateStatus(m_Status);
		//owner()->computeAcquisitionStatus(0);
		owner()->computeDetectorStatus(0);
	}
}

}
